//+------------------------------------------------------------------+
//|                                             (2009.06.19) HPB.mq4 |
//|                                                         laro_ron |
//|                                                                  |
//|  Hodrick-Prescott Filter without redrawing and Bolingen bands    |
//|                                                                  |
//| Improved HP.mq4 by gpwr from  http://codebase.mql4.com/5169      |
//|                                                                  |
//|http://en.wikipedia.org/wiki/Hodrick-Prescott_Filter              |
//|The Hodrick-Prescott filter is a mathematical tool used in        |
//|macroeconomics, especially in real business cycle theory. It is   |
//|used to obtain a smoothed non-linear representation of            |
//|a time series, one that is more sensitive to long-term than to    |
//|short-term fluctuations.                                          |
//|                                                                  |
//|This version draws the trajectory of end points of HP filter      |
//|without redrawing. It is the line of Mageta color. The line of    |
//|Khaki color (style STYLE_DOT)is the original redrawing HP filter. |
//|The lines of DarkTurquoise color are bands similar to Bolingen    |
//|bands.                                                            |
//+------------------------------------------------------------------+
#property copyright "laro_ron"
#property link      "http://en.wikipedia.org/wiki/Hodrick-Prescott_Filter"
#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Khaki
#property indicator_style1  STYLE_DOT
#property indicator_color2 Magenta
#property indicator_width2 2
#property indicator_color3 DarkTurquoise
#property indicator_color4 DarkTurquoise

//Global constants
#define pi 3.141592653589793238462643383279502884197169399375105820974944592

//Input parameters
extern int Last    =150;      // Number of bars to smooth for last bar - visible part
extern int Length =100;        // Number of filtered bars (equivalent to SMA period)
extern int AppliedPrice =0;   // Number of Applied Price
extern bool EndPoint = True;  // True = draw End Point
extern int NumBand = 25;      // Number of bars for Band SD
extern double NumSD = 2.5;    // Number of SD for Band
extern double Shift = 0;       // Shift Down / Up


double lambda; // constant as function of Length
int NBEP;      // Number of bars to smooth for End Point
int nb;        // real number of bars for Band SD
int drop;      // start drawing

//Indicator buffers
double hpf[];        // buffer HP in Last Bar (LB)
double hpf_[];       // buffer HP in the End Points (EP)
double bup[];        // buffer for band up
double bdn[];        // buffer for band down

int init()
{
   lambda=0.0625/MathPow(MathSin(pi/Length),4);      // constant
   NBEP=Length*1.5;           // for EP
   if (NumBand>NBEP) nb=NBEP ;      // for band
   else nb=NumBand;
   drop=Length*2;
   SetIndexBuffer(0,hpf);     // buffer HP in LB
   SetIndexDrawBegin(0,drop);     // start drawing from the data beginning
   SetIndexLabel(0,"HPF("+Last+","+Length+")");

   
   SetIndexBuffer(1,hpf_);     // buffer HP in the EP
   SetIndexDrawBegin(1,drop);     // start drawing from the data beginning 
   SetIndexLabel(1,"End Point("+NBEP+","+Length+")");

   SetIndexBuffer(2,bup);     // buffer for slope in the EP
   SetIndexDrawBegin(2,drop);     // start drawing from the data beginning 
   SetIndexLabel(2,"Up Band("+nb+","+DoubleToStr(NumSD,2)+")");

   SetIndexBuffer(3,bdn);     // buffer for slope in the EP
   SetIndexDrawBegin(3,drop);     // start drawing from the data beginning 
   SetIndexLabel(3,"Down Band("+nb+","+DoubleToStr(NumSD,2)+")");
   
//   IndicatorShortName("HPB("+Length+")");   //name in DataWindow

   return(0);
}
//+----------------------------------------------------------------------------------------+
int start()
{
   int limit;           // the number of changed bars
   double tmpd;
   int counted_bars=IndicatorCounted();   // the number of bars not changed after the indicator had been launched last
   double x[];                // predictors in last bar
   ArrayResize(x,Last);

   double x_[];               // predictors end points
   ArrayResize(x_,NBEP);
   double y_[];               // responses end points
   ArrayResize(y_,NBEP);

//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;      // number bars from the chart beeing repainted
   
   Comment("Bars : ",Bars,"\n","Time : ",TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS));
   
   
   if (EndPoint) {      // HP in Last Bar draws End Point curve
      for (int j=0;j<limit;j++){
         for(int i=0;i<NBEP;i++) x_[i]=GAP(AppliedPrice,j+i);       // fill predictors x_[]
         hpf_[j]=HPF(NBEP,lambda,x_,y_)+Shift;                 // calculation hpf_[]   NBEP = number counted for End Point
         if (nb>0 && NumSD>0) {                          // draws Bands
            for (i=0;i<nb;i++) x_[i]=y_[i]-GAP(AppliedPrice,j+i);   // differences
            double sd=StdD(nb, x_);                      // Standard Deviation
            bup[j]=y_[0]+Shift+NumSD*sd;        // Up Band    
            bdn[j]=y_[0]+Shift-NumSD*sd;        // Down Band
         }   
      }
   }
   
// Hodrick-Prescott Filter in Last Bar  

   for(i=0;i<Last;i++) x[i]=GAP(AppliedPrice,i);    // fill predictors x[]
   HPF(Last,lambda,x,hpf);                // calculation hpf_[]   Last = number counted

}
  
// Get Applied Price ----------------------------------------------------------------------+
double GAP(int nAppliedPrice, int nIndex)
{
   double dPrice;
   switch(nAppliedPrice){
      case 0:  dPrice=Close[nIndex];                                  break;
      case 1:  dPrice=Open[nIndex];                                   break;
      case 2:  dPrice=High[nIndex];                                   break;
      case 3:  dPrice=Low[nIndex];                                    break;
      case 4:  dPrice=(High[nIndex]+Low[nIndex])/2.0;                 break;
      case 5:  dPrice=(High[nIndex]+Low[nIndex]+Close[nIndex])/3.0;   break;
      case 6:  dPrice=(High[nIndex]+Low[nIndex]+2*Close[nIndex])/4.0; break;
      case 7:  if(Close[nIndex]>Close[nIndex+1])
                  {dPrice=High[nIndex]; break;}
               else
                  {dPrice=Low[nIndex];break;}
      default: dPrice=Close[nIndex];
   }
   return(dPrice);
}

// Standard Deviation ----------------------------------------------------------------------+
double StdD(int nb, double x[])
{
   if (nb<1) return(0.0);
   double sum1=0.0;
   double sum2=0.0;
   for (int i=0;i<nb;i++) {
      sum1+=x[i];
      sum2+=x[i]*x[i];
   }
   return(MathSqrt((sum2-MathPow((sum1/nb),2))/nb));
}

// Hodrick-Prescott Filter -----------------------------------------------------------+
double HPF(int nb, double lmbd, double x[], double& y[])
{
   double a[],b[],c[],H1,H2,H3,H4,H5,HH1,HH2,HH3,HH4,HH5,HB,HC,Z;
   ArrayResize(a,nb);
   ArrayResize(b,nb);
   ArrayResize(c,nb);
        
   a[0]=1.0+lmbd;
   b[0]=-2.0*lmbd;
   c[0]=lmbd;
   for(int i=1;i<nb-2;i++){
      a[i]=6.0*lmbd+1.0;
      b[i]=-4.0*lmbd;
      c[i]=lmbd;
   }
   a[1]=5.0*lmbd+1;
   a[nb-1]=1.0+lmbd;
   a[nb-2]=5.0*lmbd+1.0;
   b[nb-2]=-2.0*lmbd;
   b[nb-1]=0.0;
   c[nb-2]=0.0;
   c[nb-1]=0.0;
   
   //Forward
   for(i=0;i<nb;i++){
      Z=a[i]-H4*H1-HH5*HH2;
      HB=b[i];
      HH1=H1;
      H1=(HB-H4*H2)/Z;
      b[i]=H1;
      HC=c[i];
      HH2=H2;
      H2=HC/Z;
      c[i]=H2;
      a[i]=(x[i]-HH3*HH5-H3*H4)/Z;
      HH3=H3;
      H3=a[i];
      H4=HB-H5*HH1;
      HH5=H5;
      H5=HC;
   }
   
   //Backward 
   H2=0;
   H1=a[nb-1];
   y[nb-1]=H1;
   for(i=nb-2;i>=0;i--){
      y[i]=a[i]-b[i]*H1-c[i]*H2;
      H2=H1;
      H1=y[i];
   }
   return(y[0]);
}

